<?php

namespace Pinpoint\SalesRep\Plugin;

use Magento\Backend\Model\Auth\Session;
use Magento\Framework\App\Request\Http;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\Order\Email\Sender\OrderCommentSender;
use Magento\Sales\Model\Order\Status\History;
use Magento\User\Model\ResourceModel\User\CollectionFactory as userCollectionFactory;
use Magento\User\Model\User;
use Pinpoint\SalesRep\Helper\Data;
use Pinpoint\SalesRep\Model\ResourceModel\SalesRepsList\CollectionFactory;
use Pinpoint\SalesRep\Model\SalesRepFactory;

/**
 * Class AdminAddComment
 */
class AdminAddComment
{
    /**
     * @var integer
     */
    const DEFAULT_SALESREP_ID = 0;

    /**
     * @var Http
     */
    protected Http $http;

    /**
     * @var Order
     */
    protected Order $order;

    /**
     * @var Session
     */
    protected Session $session;

    /**
     * @var integer
     */
    protected int $currentUserId;

    /**
     * @var CollectionFactory
     */
    protected CollectionFactory $collectionFactory;

    /**
     * @var SalesRepFactory
     */
    protected SalesRepFactory $salesRepModel;

    /**
     * @var OrderCommentSender
     */
    protected OrderCommentSender $orderCommentSender;

    /**
     * @var Data
     */
    protected Data $data;

    /**
     * @var userCollectionFactory
     */
    protected userCollectionFactory $userCollectionFactory;

    /**
     * View constructor.
     * @param Http $http
     * @param Order $order
     * @param Session $session
     * @param SalesRepFactory $salesRepModel
     * @param CollectionFactory $collectionFactory
     * @param OrderCommentSender $orderCommentSender
     * @param Data $data
     * @param userCollectionFactory $userCollectionFactory
     */
    public function __construct(
        Http $http,
        Order $order,
        Session $session,
        SalesRepFactory $salesRepModel,
        CollectionFactory $collectionFactory,
        OrderCommentSender $orderCommentSender,
        Data $data,
        userCollectionFactory $userCollectionFactory
    ) {
        $this->http = $http;
        $this->order = $order;
        $this->session = $session;
        $this->collectionFactory = $collectionFactory;
        $this->salesRepModel = $salesRepModel;
        $this->orderCommentSender = $orderCommentSender;
        $this->data = $data;
        $this->userCollectionFactory = $userCollectionFactory;
    }

    /**
     * Around setComment to assign sales rep
     *
     * @param History $subject
     * @param callable $proceed
     * @param $comment
     * @return callable
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\MailException
     */
    public function aroundSetComment(History $subject, callable $proceed, $comment): callable
    {

        // Get Order Instances
        $order_id = $this->http->getParam('order_id');
        $order = $this->order->load($order_id);

        // Check if this order has a sales representative
        $salesRepId = $this->checkSalesRep($order_id);
        if (!$salesRepId) {

            // Does not have sales representative, lets assign then send an email
            $this->currentUser = $this->getCurrentAdminUser();
            $this->assignSalesRep($order_id, $this->currentUser->getId());
        } else {

            // A status change detected, send email to the current salesrep
            $this->sendEmailToSalesRep($salesRepId, $order, $comment);
        }

        return $proceed($comment);
    }

    /**
     * Get current admin user
     *
     * @return \Magento\User\Model\User|null
     */
    public function getCurrentAdminUser(): ?User
    {
        return $this->session->getUser();
    }

    /**
     * Check if a sales representative is assigned and returns user ID
     *
     * @param $order_id
     * @return int|string
     */
    public function checkSalesRep($order_id)
    {
        $salesRepId = self::DEFAULT_SALESREP_ID;
        $collection = $this->collectionFactory->create();
        $collection->addFieldToSelect('user_id')->addFieldToFilter('sales_order_entity_id', $order_id)->load();
        $collection->getColumnValues('user_id');
        $data = $collection->getData();
        if ($data) {
            $salesRepId = $data[0]['user_id'];
        }

        return $salesRepId;
    }

    /**
     * Assign sales representative to order
     *
     * @param $order_id
     * @param $user_id
     * @return boolean
     * @throws \Exception
     */
    public function assignSalesRep($order_id, $user_id): bool
    {
        $newModel = $this->salesRepModel->create();
        $newModel->setData('sales_order_entity_id', $order_id);
        $newModel->setData('user_id', $user_id);
        $newModel->save();
        return true;
    }

    /**
     * Sends an email to sales representative
     *
     * @param $user_id
     * @param $order
     * @param $comment
     * @return void
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\MailException
     */
    public function sendEmailToSalesRep($user_id, $order, $comment): void
    {
        foreach ($this->userCollectionFactory->create() as $user) {
            if ($user->getId() == $user_id) {
                $this->data->sendMail($user->getName(), $user->getEmail(), $order, $comment);
                break;
            }
        }
    }
}
